#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Device File Generator
"""
import os
import sys

sys.path.append(os.path.join("..", ".."))

__author__ = "Niklas Hauser"
__copyright__ = "Niklas Hauser"
__credits__ = ["Niklas Hauser"]
__license__ = "Mozilla Public License 2.0"
__version__ = "0.1"
__maintainer__ = "Niklas Hauser"
__email__ = "niklas.hauser@rwth-aachen.de"
__status__ = "Pre-Alpha"
